@@ -33,13 +33,13 @@
 
 #define RSORT(x) (SortAlias & SORT_REVERSE) ? -x : x
 
-static struct mapping_t AliasHelp[] = {
+static const struct mapping_t AliasHelp[] = {
   { N_("Exit"),   OP_EXIT },
   { N_("Del"),    OP_DELETE },
   { N_("Undel"),  OP_UNDELETE },
   { N_("Select"), OP_GENERIC_SELECT_ENTRY },
   { N_("Help"),   OP_HELP },
-  { NULL }
+  { NULL,	  0 }
 };
 
 static const char *
