@@ -110,6 +110,83 @@ static const char *No_visible = N_("No visible messages.");
 
 extern size_t UngetCount;
 
+/* de facto standard escapes for tsl/fsl */
+static char *tsl = "\033]0;";
+static char *fsl = "\007";
+
+/* terminal status capability check. terminfo must have been initialized. */
+short mutt_ts_capability(void)
+{
+  char *term = getenv("TERM");
+  char *tcaps;
+  int tcapi;
+  char **termp;
+  char *known[] = {
+    "color-xterm",
+    "cygwin",
+    "eterm",
+    "kterm",
+    "nxterm",
+    "putty",
+    "rxvt",
+    "screen",
+    "xterm",
+    NULL
+  };
+
+  /* If tsl is set, then terminfo says that status lines work. */
+  tcaps = tigetstr("tsl");
+  if (tcaps && tcaps != (char *)-1 && *tcaps)
+  {
+    /* update the static defns of tsl/fsl from terminfo */
+    tsl = safe_strdup(tcaps);
+
+    tcaps = tigetstr("fsl");
+    if (tcaps && tcaps != (char *)-1 && *tcaps)
+      fsl = safe_strdup(tcaps);
+
+    return 1;
+  }
+
+  /* If XT (boolean) is set, then this terminal supports the standard escape. */
+  /* Beware: tigetflag returns -1 if XT is invalid or not a boolean. */
+#ifdef HAVE_USE_EXTENDED_NAMES
+  use_extended_names (TRUE);
+  tcapi = tigetflag("XT");
+  if (tcapi == 1)
+    return 1;
+#endif /* HAVE_USE_EXTENDED_NAMES */
+
+  /* Check term types that are known to support the standard escape without
+   * necessarily asserting it in terminfo. */
+  for (termp = known; termp; termp++)
+  {
+    if (term && *termp && mutt_strncasecmp (term, *termp, strlen(*termp)))
+      return 1;
+  }
+
+  /* not supported */
+  return 0;
+}
+
+void mutt_ts_status(char *str)
+{
+  /* If empty, do not set.  To clear, use a single space. */
+  if (str == NULL || *str == '\0')
+    return;
+  fprintf(stderr, "%s%s%s", tsl, str, fsl);
+}
+
+void mutt_ts_icon(char *str)
+{
+  /* If empty, do not set.  To clear, use a single space. */
+  if (str == NULL || *str == '\0')
+    return;
+
+  /* icon setting is not supported in terminfo, so hardcode the escape - yuck */
+  fprintf(stderr, "\033]1;%s\007", str);
+}
+
 void index_make_entry (char *s, size_t l, MUTTMENU *menu, int num)
 {
   format_flag flag = M_FORMAT_MAKEPRINT | M_FORMAT_ARROWCURSOR | M_FORMAT_INDEX;
@@ -401,7 +478,7 @@ static void resort_index (MUTTMENU *menu)
   menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
 }
 
-static struct mapping_t IndexHelp[] = {
+static const struct mapping_t IndexHelp[] = {
   { N_("Quit"),  OP_QUIT },
   { N_("Del"),   OP_DELETE },
   { N_("Undel"), OP_UNDELETE },
@@ -473,10 +550,6 @@ int mutt_index_menu (void)
        * modified underneath us.)
        */
 
-#ifdef USE_IMAP
-      imap_allow_reopen (Context);
-#endif
-
       index_hint = (Context->vcount && menu->current >= 0 && menu->current < Context->vcount) ? CURHDR->index : 0;
 
       if ((check = mx_check_mailbox (Context, &index_hint, 0)) < 0)
@@ -559,13 +632,18 @@ int mutt_index_menu (void)
       if (menu->redraw & REDRAW_STATUS)
       {
 	menu_status_line (buf, sizeof (buf), menu, NONULL (Status));
-	CLEARLINE (option (OPTSTATUSONTOP) ? 0 : LINES-2);
+	move (option (OPTSTATUSONTOP) ? 0 : LINES-2, 0);
 	SETCOLOR (MT_COLOR_STATUS);
-        BKGDSET (MT_COLOR_STATUS);
 	mutt_paddstr (COLS, buf);
-	SETCOLOR (MT_COLOR_NORMAL);
-        BKGDSET (MT_COLOR_NORMAL);
+	NORMAL_COLOR;
 	menu->redraw &= ~REDRAW_STATUS;
+	if (option(OPTTSENABLED) && TSSupported)
+	{
+	  menu_status_line (buf, sizeof (buf), menu, NONULL (TSStatusFormat));
+	  mutt_ts_status(buf);
+	  menu_status_line (buf, sizeof (buf), menu, NONULL (TSIconFormat));
+	  mutt_ts_icon(buf);
+	}
       }
 
       menu->redraw = 0;
@@ -686,10 +764,6 @@ int mutt_index_menu (void)
       mutt_curs_set (1);	/* fallback from the pager */
     }
 
-#ifdef USE_IMAP
-    imap_disallow_reopen (Context);
-#endif
-
     switch (op)
     {
 
@@ -922,6 +996,11 @@ int mutt_index_menu (void)
 	    resort_index (menu);
 	    set_option (OPTSEARCHINVALID);
 	  }
+	  if (menu->menu == MENU_PAGER)
+	  {
+	    op = OP_DISPLAY_MESSAGE;
+	    continue;
+	  }
 	  menu->redraw |= REDRAW_STATUS;
 	}
 	break;
@@ -991,6 +1070,23 @@ int mutt_index_menu (void)
 	if (Context && Context->magic == M_IMAP)
 	  imap_check_mailbox (Context, &index_hint, 1);
         break;
+
+      case OP_MAIN_IMAP_LOGOUT_ALL:
+	if (Context && Context->magic == M_IMAP)
+	{
+	  if (mx_close_mailbox (Context, &index_hint) != 0)
+	  {
+	    set_option (OPTSEARCHINVALID);
+	    menu->redraw = REDRAW_FULL;
+	    break;
+	  }
+	  FREE (&Context);
+	}
+	imap_logout_all();
+	mutt_message _("Logged out of IMAP servers.");
+	set_option (OPTSEARCHINVALID);
+	menu->redraw = REDRAW_FULL;
+	break;
 #endif
 
       case OP_MAIN_SYNC_FOLDER:
@@ -999,7 +1095,6 @@ int mutt_index_menu (void)
 	  break;
 
 	CHECK_MSGCOUNT;
-        CHECK_VISIBLE;
 	CHECK_READONLY;
 	{
 	  int oldvcount = Context->vcount;
@@ -1007,15 +1102,19 @@ int mutt_index_menu (void)
 	  int check, newidx;
 	  HEADER *newhdr = NULL;
 
-	  /* threads may be reordered, so figure out what header the cursor
-	   * should be on. #3092 */
-	  newidx = menu->current;
-	  if (CURHDR->deleted)
-	    newidx = ci_next_undeleted (menu->current);
-	  if (newidx < 0)
-	    newidx = ci_previous_undeleted (menu->current);
-	  if (newidx >= 0)
-	    newhdr = Context->hdrs[Context->v2r[newidx]];
+	  /* don't attempt to move the cursor if there are no visible messages in the current limit */
+	  if (menu->current < Context->vcount)
+	  {
+	    /* threads may be reordered, so figure out what header the cursor
+	     * should be on. #3092 */
+	    newidx = menu->current;
+	    if (CURHDR->deleted)
+	      newidx = ci_next_undeleted (menu->current);
+	    if (newidx < 0)
+	      newidx = ci_previous_undeleted (menu->current);
+	    if (newidx >= 0)
+	      newhdr = Context->hdrs[Context->v2r[newidx]];
+	  }
 
 	  if ((check = mx_sync_mailbox (Context, &index_hint)) == 0)
 	  {
@@ -1111,6 +1210,10 @@ int mutt_index_menu (void)
 	}
 	mutt_str_replace (&CurrentFolder, buf);
 
+	/* keepalive failure in mutt_enter_fname may kill connection. #3028 */
+	if (Context && !Context->path)
+	  FREE (&Context);
+
         if (Context)
         {
 	  int check;
@@ -1220,7 +1323,7 @@ int mutt_index_menu (void)
 
         if ((Sort & SORT_MASK) != SORT_THREADS)
 	  mutt_error _("Threading is not enabled.");
-	else
+	else if (CURHDR->env->in_reply_to || CURHDR->env->references)
 	{
 	  {
 	    HEADER *oldcur = CURHDR;
@@ -1241,8 +1344,10 @@ int mutt_index_menu (void)
 	  else
 	    menu->redraw |= REDRAW_INDEX;
 	}
+	else
+	  mutt_error _("Thread cannot be broken, message is not part of a thread");
 
-	  break;
+	break;
 
       case OP_MAIN_LINK_THREADS:
 
@@ -2033,6 +2138,11 @@ int mutt_index_menu (void)
 	    if ((menu->current = (op == OP_MAIN_READ_THREAD ?
 				  mutt_next_thread (CURHDR) : mutt_next_subthread (CURHDR))) == -1)
 	      menu->current = menu->oldcurrent;
+	    else if (menu->menu == MENU_PAGER)
+	    {
+	      op = OP_DISPLAY_MESSAGE;
+	      continue;
+	    }
 	  }
 	  menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
 	}
