@@ -550,21 +550,33 @@ mutt_hcache_per_folder(const char *path, const char *folder,
   {
     md5_buffer (folder, strlen (folder), &md5sum);
 
+    /* On some systems (e.g. OS X), snprintf is defined as a macro.
+     * Embedding directives inside macros is undefined, so we have to duplicate
+     * the whole call:
+     */
+#ifndef HAVE_ICONV
     ret = snprintf(hcpath, _POSIX_PATH_MAX,
                    "%s/%02x%02x%02x%02x%02x%02x%02x%02x"
                    "%02x%02x%02x%02x%02x%02x%02x%02x"
-#ifndef HAVE_ICONV
 		   "-%s"
-#endif
 		   ,
 		   path, md5sum[0], md5sum[1], md5sum[2], md5sum[3],
                    md5sum[4], md5sum[5], md5sum[6], md5sum[7], md5sum[8],
                    md5sum[9], md5sum[10], md5sum[11], md5sum[12],
                    md5sum[13], md5sum[14], md5sum[15]
-#ifndef HAVE_ICONV
 		   ,chs
-#endif
 		   );
+#else
+    ret = snprintf(hcpath, _POSIX_PATH_MAX,
+                   "%s/%02x%02x%02x%02x%02x%02x%02x%02x"
+                   "%02x%02x%02x%02x%02x%02x%02x%02x"
+		   ,
+		   path, md5sum[0], md5sum[1], md5sum[2], md5sum[3],
+                   md5sum[4], md5sum[5], md5sum[6], md5sum[7], md5sum[8],
+                   md5sum[9], md5sum[10], md5sum[11], md5sum[12],
+                   md5sum[13], md5sum[14], md5sum[15]
+		   );
+#endif
   }
   
   if (ret <= 0)
@@ -911,12 +923,15 @@ static int
 hcache_open_tc (struct header_cache* h, const char* path)
 {
   h->db = tcbdbnew();
+  if (!h->db)
+      return -1;
   if (option(OPTHCACHECOMPRESS))
     tcbdbtune(h->db, 0, 0, 0, -1, -1, BDBTDEFLATE);
   if (tcbdbopen(h->db, path, BDBOWRITER | BDBOCREAT))
     return 0;
   else
   {
+    dprint(2, (debugfile, "tcbdbopen %s failed: %s\n", path, errno));
     tcbdbdel(h->db);
     return -1;
   }
@@ -928,7 +943,8 @@ mutt_hcache_close(header_cache_t *h)
   if (!h)
     return;
 
-  tcbdbclose(h->db);
+  if (!tcbdbclose(h->db))
+      dprint (2, (debugfile, "tcbdbclose failed for %s: %d\n", h->folder, errno));
   tcbdbdel(h->db);
   FREE(&h->folder);
   FREE(&h);
@@ -1131,7 +1147,10 @@ mutt_hcache_open(const char *path, const char *folder, hcache_namer_t namer)
 
   /* Calculate the current hcache version from dynamic configuration */
   if (hcachever == 0x0) {
-    unsigned char digest[16];
+    union {
+      unsigned char charval[16];
+      unsigned int intval;
+    } digest;
     struct md5_ctx ctx;
     SPAM_LIST *spam;
     RX_LIST *nospam;
@@ -1157,8 +1176,8 @@ mutt_hcache_open(const char *path, const char *folder, hcache_namer_t namer)
     }
 
     /* Get a hash and take its bytes as an (unsigned int) hash version */
-    md5_finish_ctx(&ctx, digest);
-    hcachever = *((unsigned int *)digest);
+    md5_finish_ctx(&ctx, digest.charval);
+    hcachever = digest.intval;
   }
 
   h->db = NULL;
