@@ -27,7 +27,6 @@
 
 #include "mutt.h"
 #include "imap_private.h"
-#include "message.h"
 #include "mx.h"
 #include "buffy.h"
 
@@ -53,7 +52,7 @@ static void cmd_parse_myrights (IMAP_DATA* idata, const char* s);
 static void cmd_parse_search (IMAP_DATA* idata, const char* s);
 static void cmd_parse_status (IMAP_DATA* idata, char* s);
 
-static char *Capabilities[] = {
+static const char * const Capabilities[] = {
   "IMAP4",
   "IMAP4rev1",
   "STATUS",
@@ -233,9 +232,6 @@ int imap_exec (IMAP_DATA* idata, const char* cmdstr, int flags)
   int rc;
 
   if ((rc = cmd_start (idata, cmdstr, flags)) < 0)
-    return rc;
-
-  if (rc < 0)
   {
     cmd_handle_fatal (idata);
     return -1;
@@ -253,7 +249,7 @@ int imap_exec (IMAP_DATA* idata, const char* cmdstr, int flags)
 
   if (rc != IMAP_CMD_OK)
   {
-    if (flags & IMAP_CMD_FAIL_OK && idata->status != IMAP_FATAL)
+    if ((flags & IMAP_CMD_FAIL_OK) && idata->status != IMAP_FATAL)
       return -2;
 
     dprint (1, (debugfile, "imap_exec: command failed: %s\n", idata->buf));
@@ -298,7 +294,7 @@ void imap_cmd_finish (IMAP_DATA* idata)
       dprint (2, (debugfile, "imap_cmd_finish: Expunging mailbox\n"));
       imap_expunge_mailbox (idata);
       /* Detect whether we've gotten unexpected EXPUNGE messages */
-      if (idata->reopen & IMAP_EXPUNGE_PENDING &&
+      if ((idata->reopen & IMAP_EXPUNGE_PENDING) &&
 	  !(idata->reopen & IMAP_EXPUNGE_EXPECTED))
 	idata->check_status = IMAP_EXPUNGE_PENDING;
       idata->reopen &= ~(IMAP_EXPUNGE_PENDING | IMAP_NEWMAIL_PENDING |
@@ -765,7 +761,8 @@ static void cmd_parse_lsub (IMAP_DATA* idata, char* s)
   idata->cmddata = &list;
   cmd_parse_list (idata, s);
   idata->cmddata = NULL;
-  if (!list.name)
+  /* noselect is for a gmail quirk (#3445) */
+  if (!list.name || list.noselect)
     return;
 
   dprint (3, (debugfile, "Subscribing to %s\n", list.name));
@@ -780,7 +777,8 @@ static void cmd_parse_lsub (IMAP_DATA* idata, char* s)
     url.user = NULL;
   url_ciss_tostring (&url, buf + 11, sizeof (buf) - 10, 0);
   safe_strcat (buf, sizeof (buf), "\"");
-  memset (&token, 0, sizeof (token));
+  mutt_buffer_init (&token);
+  mutt_buffer_init (&err);
   err.data = errstr;
   err.dsize = sizeof (errstr);
   if (mutt_parse_rc_line (buf, &token, &err))
@@ -993,14 +991,19 @@ static void cmd_parse_status (IMAP_DATA* idata, char* s)
         dprint (3, (debugfile, "Found %s in buffy list (OV: %d ON: %d U: %d)\n",
                     mailbox, olduv, oldun, status->unseen));
         
-        if (olduv && olduv == status->uidvalidity)
-        {
-          if (oldun < status->uidnext)
-            inc->new = status->unseen;
-        }
-        else if (!olduv && !oldun)
-	  /* first check per session, use recent. might need a flag for this. */
-	  inc->new = status->recent;
+	if (option(OPTMAILCHECKRECENT))
+	{
+	  if (olduv && olduv == status->uidvalidity)
+	  {
+	    if (oldun < status->uidnext)
+	      inc->new = status->unseen;
+	  }
+	  else if (!olduv && !oldun)
+	    /* first check per session, use recent. might need a flag for this. */
+	    inc->new = status->recent;
+	  else
+	    inc->new = status->unseen;
+	}
 	else
           inc->new = status->unseen;
 
