@@ -30,7 +30,6 @@
 #include "globals.h"
 #include "sort.h"
 #include "browser.h"
-#include "message.h"
 #include "imap_private.h"
 #if defined(USE_SSL)
 # include "mutt_ssl.h"
@@ -281,7 +280,7 @@ void imap_expunge_mailbox (IMAP_DATA* idata)
 	FREE (&idata->cache[cacheno].path);
       }
 
-      imap_free_header_data (&h->data);
+      imap_free_header_data ((IMAP_HEADER_DATA**)&h->data);
     }
   }
 
@@ -374,6 +373,7 @@ IMAP_DATA* imap_conn_find (const ACCOUNT* account, int flags)
     if (!imap_authenticate (idata))
     {
       idata->state = IMAP_AUTHENTICATED;
+      FREE (&idata->capstr);
       new = 1;
       if (idata->conn->ssf)
 	dprint (2, (debugfile, "Communication encrypted at %d bits\n",
@@ -381,8 +381,6 @@ IMAP_DATA* imap_conn_find (const ACCOUNT* account, int flags)
     }
     else
       mutt_account_unsetpass (&idata->conn->account);
-
-    FREE (&idata->capstr);
   }
   if (new && idata->state == IMAP_AUTHENTICATED)
   {
@@ -555,7 +553,6 @@ static char* imap_get_flags (LIST** hflags, char* s)
 
 int imap_open_mailbox (CONTEXT* ctx)
 {
-  CONNECTION *conn;
   IMAP_DATA *idata;
   IMAP_STATUS* status;
   char buf[LONG_STRING];
@@ -576,8 +573,6 @@ int imap_open_mailbox (CONTEXT* ctx)
   if (idata->state < IMAP_AUTHENTICATED)
     goto fail;
 
-  conn = idata->conn;
-
   /* once again the context is new */
   ctx->data = idata;
   ctx->mx_close = imap_close_mailbox;
@@ -774,7 +769,6 @@ int imap_open_mailbox (CONTEXT* ctx)
 
 int imap_open_mailbox_append (CONTEXT *ctx)
 {
-  CONNECTION *conn;
   IMAP_DATA *idata;
   char buf[LONG_STRING];
   char mailbox[LONG_STRING];
@@ -793,8 +787,6 @@ int imap_open_mailbox_append (CONTEXT *ctx)
     return -1;
   }
 
-  conn = idata->conn;
-
   ctx->magic = M_IMAP;
   ctx->data = idata;
 
@@ -969,7 +961,7 @@ int imap_exec_msgset (IMAP_DATA* idata, const char* pre, const char* post,
   int rc;
   int count = 0;
 
-  if (! (cmd = mutt_buffer_init (NULL)))
+  if (! (cmd = mutt_buffer_new ()))
   {
     dprint (1, (debugfile, "imap_exec_msgset: unable to allocate buffer\n"));
     return -1;
@@ -1288,8 +1280,18 @@ int imap_sync_mailbox (CONTEXT* ctx, int expunge, int* index_hint)
     goto out;
   }
 
+  /* Update local record of server state to reflect the synchronization just
+   * completed.  imap_read_headers always overwrites hcache-origin flags, so
+   * there is no need to mutate the hcache after flag-only changes. */
   for (n = 0; n < ctx->msgcount; n++)
+  {
+    HEADER_DATA(ctx->hdrs[n])->deleted = ctx->hdrs[n]->deleted;
+    HEADER_DATA(ctx->hdrs[n])->flagged = ctx->hdrs[n]->flagged;
+    HEADER_DATA(ctx->hdrs[n])->old = ctx->hdrs[n]->old;
+    HEADER_DATA(ctx->hdrs[n])->read = ctx->hdrs[n]->read;
+    HEADER_DATA(ctx->hdrs[n])->replied = ctx->hdrs[n]->replied;
     ctx->hdrs[n]->changed = 0;
+  }
   ctx->changed = 0;
 
   /* We must send an EXPUNGE command if we're not closing. */
@@ -1359,7 +1361,7 @@ int imap_close_mailbox (CONTEXT* ctx)
   for (i = 0; i < ctx->msgcount; i++)
     /* mailbox may not have fully loaded */
     if (ctx->hdrs[i] && ctx->hdrs[i]->data)
-      imap_free_header_data (&(ctx->hdrs[i]->data));
+      imap_free_header_data ((IMAP_HEADER_DATA**)&(ctx->hdrs[i]->data));
 
   for (i = 0; i < IMAP_CACHE_LEN; i++)
   {
@@ -1794,7 +1796,7 @@ int imap_search (CONTEXT* ctx, const pattern_t* pat)
   if (!do_search (pat, 1))
     return 0;
 
-  memset (&buf, 0, sizeof (buf));
+  mutt_buffer_init (&buf);
   mutt_buffer_addstr (&buf, "UID SEARCH ");
   if (imap_compile_search (pat, &buf) < 0)
   {
@@ -1813,7 +1815,6 @@ int imap_search (CONTEXT* ctx, const pattern_t* pat)
 
 int imap_subscribe (char *path, int subscribe)
 {
-  CONNECTION *conn;
   IMAP_DATA *idata;
   char buf[LONG_STRING];
   char mbox[LONG_STRING];
@@ -1829,15 +1830,14 @@ int imap_subscribe (char *path, int subscribe)
   if (!(idata = imap_conn_find (&(mx.account), 0)))
     goto fail;
 
-  conn = idata->conn;
-
   imap_fix_path (idata, mx.mbox, buf, sizeof (buf));
   if (!*buf)
     strfcpy (buf, "INBOX", sizeof (buf));
 
   if (option (OPTIMAPCHECKSUBSCRIBED))
   {
-    memset (&token, 0, sizeof (token));
+    mutt_buffer_init (&token);
+    mutt_buffer_init (&err);
     err.data = errstr;
     err.dsize = sizeof (errstr);
     snprintf (mbox, sizeof (mbox), "%smailboxes \"%s\"",
@@ -1941,7 +1941,6 @@ imap_complete_hosts (char *dest, size_t len)
 /* imap_complete: given a partial IMAP folder path, return a string which
  *   adds as much to the path as is unique */
 int imap_complete(char* dest, size_t dlen, char* path) {
-  CONNECTION* conn;
   IMAP_DATA* idata;
   char list[LONG_STRING];
   char buf[LONG_STRING];
@@ -1966,7 +1965,6 @@ int imap_complete(char* dest, size_t dlen, char* path) {
     strfcpy (dest, path, dlen);
     return imap_complete_hosts (dest, dlen);
   }
-  conn = idata->conn;
 
   /* reformat path for IMAP list, and append wildcard */
   /* don't use INBOX in place of "" */
