@@ -60,7 +60,6 @@ static char* msg_parse_flags (IMAP_HEADER* h, char* s);
 int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
 {
   CONTEXT* ctx;
-  char buf[LONG_STRING];
   char *hdrreq = NULL;
   FILE *fp;
   char tempfile[_POSIX_PATH_MAX];
@@ -75,6 +74,7 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
   int retval = -1;
 
 #if USE_HCACHE
+  char buf[LONG_STRING];
   unsigned int *uid_validity = NULL;
   unsigned int *puidnext = NULL;
   unsigned int uidnext = 0;
@@ -547,7 +547,7 @@ int imap_fetch_message (MESSAGE *msg, CONTEXT *ctx, int msgno)
    * IMAP server doesn't know the message has been \Seen. So we capture
    * the server's notion of 'read' and if it differs from the message info
    * picked up in mutt_read_rfc822_header, we mark the message (and context
-   * changed). Another possiblity: ignore Status on IMAP?*/
+   * changed). Another possibility: ignore Status on IMAP?*/
   read = h->read;
   newenv = mutt_read_rfc822_header (msg->fp, h, 0, 0);
   mutt_merge_envelopes(h->env, &newenv);
@@ -601,6 +601,7 @@ int imap_append_message (CONTEXT *ctx, MESSAGE *msg)
   char mbox[LONG_STRING];
   char mailbox[LONG_STRING];
   char internaldate[IMAP_DATELEN];
+  char imap_flags[SHORT_STRING];
   size_t len;
   progress_t progressbar;
   size_t sent;
@@ -643,12 +644,19 @@ int imap_append_message (CONTEXT *ctx, MESSAGE *msg)
 
   imap_munge_mbox_name (mbox, sizeof (mbox), mailbox);
   imap_make_date (internaldate, msg->received);
-  snprintf (buf, sizeof (buf), "APPEND %s (%s%s%s%s%s) \"%s\" {%lu}", mbox,
-	    msg->flags.read    ? "\\Seen"      : "",
-	    msg->flags.read && (msg->flags.replied || msg->flags.flagged) ? " " : "",
-	    msg->flags.replied ? "\\Answered" : "",
-	    msg->flags.replied && msg->flags.flagged ? " " : "",
-	    msg->flags.flagged ? "\\Flagged"  : "",
+
+  imap_flags[0] = imap_flags[1] = 0;
+  if (msg->flags.read)
+    safe_strcat (imap_flags, sizeof (imap_flags), " \\Seen");
+  if (msg->flags.replied)
+    safe_strcat (imap_flags, sizeof (imap_flags), " \\Answered");
+  if (msg->flags.flagged)
+    safe_strcat (imap_flags, sizeof (imap_flags), " \\Flagged");
+  if (msg->flags.draft)
+    safe_strcat (imap_flags, sizeof (imap_flags), " \\Draft");
+
+  snprintf (buf, sizeof (buf), "APPEND %s (%s) \"%s\" {%lu}", mbox,
+            imap_flags + 1,
 	    internaldate,
 	    (unsigned long) len);
 
