@@ -690,7 +690,6 @@ static int mbox_has_new(CONTEXT *ctx)
 void mbox_reset_atime (CONTEXT *ctx, struct stat *st)
 {
   struct utimbuf utimebuf;
-  int i;
   struct stat _st;
 
   if (!st)
@@ -705,7 +704,7 @@ void mbox_reset_atime (CONTEXT *ctx, struct stat *st)
 
   /*
    * When $mbox_check_recent is set, existing new mail is ignored, so do not
-   * recent the atime to mtime-1 to signal new mail.
+   * reset the atime to mtime-1 to signal new mail.
    */
   if (!option(OPTMAILCHECKRECENT) && utimebuf.actime >= utimebuf.modtime && mbox_has_new(ctx))
     utimebuf.actime = utimebuf.modtime - 1;
@@ -979,7 +978,7 @@ int mbox_sync_mailbox (CONTEXT *ctx, int *index_hint)
 
   if (fclose (ctx->fp) != 0 || i == -1)
   {
-    /* error occured while writing the mailbox back, so keep the temp copy
+    /* error occurred while writing the mailbox back, so keep the temp copy
      * around
      */
     
