@@ -48,15 +48,16 @@ enum
 {
   DISPINLINE,
   DISPATTACH,
-  DISPFORMDATA
+  DISPFORMDATA,
+  DISPNONE /* no preferred disposition */
 };
 
 /* MIME encoding/decoding global vars */
 
 #ifndef _SENDLIB_C
-extern int Index_hex[];
-extern int Index_64[];
-extern char B64Chars[];
+extern const int Index_hex[];
+extern const int Index_64[];
+extern const char B64Chars[];
 #endif
 
 #define hexval(c) Index_hex[(unsigned int)(c)]
