@@ -1213,7 +1213,7 @@ static int imap_open_new_message (MESSAGE *msg, CONTEXT *dest, HEADER *hdr)
 #endif
 
 /* args:
- *	dest	destintation mailbox
+ *	dest	destination mailbox
  *	hdr	message being copied (required for maildir support, because
  *		the filename depends on the message flags)
  */
@@ -1255,6 +1255,7 @@ MESSAGE *mx_open_new_message (CONTEXT *dest, HEADER *hdr, int flags)
     msg->flags.flagged = hdr->flagged;
     msg->flags.replied = hdr->replied;
     msg->flags.read    = hdr->read;
+    msg->flags.draft   = (flags & M_SET_DRAFT) ? 1 : 0;
     msg->received = hdr->received;
   }
 
