@@ -34,6 +34,7 @@ pgp_sig_t;
 struct pgp_keyinfo
 {
   char *keyid;
+  char *fingerprint;
   struct pgp_uid *address;
   int flags;
   short keylen;
@@ -43,13 +44,6 @@ struct pgp_keyinfo
   struct pgp_keyinfo *parent;
   struct pgp_signature *sigs;
   struct pgp_keyinfo *next;
-
-  short fp_len;			  /* length of fingerprint.
-				   * 20 for sha-1, 16 for md5.
-				   */
-  unsigned char fingerprint[20];  /* large enough to hold SHA-1 and RIPEMD160
-                                     hashes (20 bytes), MD5 hashes just use the
-                                     first 16 bytes */
 };
 /* Note, that pgp_key_t is now pointer and declared in crypt.h */
 
